@@ -59,8 +59,8 @@ API Reference
 #include "sqlite3.h"
 #endif
 
-#if SQLITE_VERSION_NUMBER < 3013000
-#error Your SQLite version is too old.  It must be at least 3.13
+#if SQLITE_VERSION_NUMBER < 3024000
+#error Your SQLite version is too old.  It must be at least 3.24
 #endif
 
 /* system headers */
@@ -114,7 +114,7 @@ static int APSW_Should_Fault(const char *);
 /* The module object */
 static PyObject *apswmodule;
 
-/* Everything except the module itself is in seperate files */
+/* Everything except the module itself is in separate files */
 #ifdef PYPY_VERSION
 #include "pypycompat.c"
 #endif
@@ -984,6 +984,41 @@ get_compile_options(void)
   return NULL;
 }
 
+/** .. attribute:: keywords
+
+    A set containing every SQLite keyword
+
+    -* sqlite3_keyword_count sqlite3_keyword_name
+
+*/
+static PyObject*
+get_keywords(void)
+{
+  int i, j, count, size;
+  PyObject *res=NULL, *tmpstring;
+  const char *name;
+
+  res=PySet_New(0);
+  if(!res) goto fail;
+
+  count=sqlite3_keyword_count();  /* No PYSQLITE_CALL needed */
+  for(i=0; i<count; i++)
+  {
+      j=sqlite3_keyword_name(i, &name, &size); /* No PYSQLITE_CALL needed */
+      assert(j==SQLITE_OK);
+      tmpstring=convertutf8stringsize(name, size);
+      if(!tmpstring) goto fail;
+      j=PySet_Add(res, tmpstring);
+      Py_DECREF(tmpstring);
+      if(j) goto fail;
+  }
+
+  return res;
+ fail:
+  Py_XDECREF(res);
+  return NULL;
+}
+
 
 /** .. method:: format_sql_value(value) -> string
 
@@ -1365,7 +1400,7 @@ modules etc. For example::
 #define ADDINT(n) {#n, n}
 
     static const
-    struct { const char *name; int value; } integers[]={
+    struct { const char *name; const int value; } integers[]={
       DICT("mapping_authorizer_return"),
       ADDINT(SQLITE_DENY),
       ADDINT(SQLITE_IGNORE),
@@ -1420,6 +1455,11 @@ modules etc. For example::
       ADDINT(SQLITE_INDEX_CONSTRAINT_LIKE),
       ADDINT(SQLITE_INDEX_CONSTRAINT_REGEXP),
       ADDINT(SQLITE_INDEX_CONSTRAINT_GLOB),
+      ADDINT(SQLITE_INDEX_CONSTRAINT_ISNULL),
+      ADDINT(SQLITE_INDEX_CONSTRAINT_ISNOT),
+      ADDINT(SQLITE_INDEX_CONSTRAINT_ISNOTNULL),
+      ADDINT(SQLITE_INDEX_CONSTRAINT_IS),
+      ADDINT(SQLITE_INDEX_CONSTRAINT_NE),
       END,
 
       /* extended result codes */
@@ -1479,6 +1519,16 @@ modules etc. For example::
       ADDINT(SQLITE_AUTH_USER),
       ADDINT(SQLITE_IOERR_VNODE),
       ADDINT(SQLITE_IOERR_AUTH),
+      ADDINT(SQLITE_OK_LOAD_PERMANENTLY),
+      ADDINT(SQLITE_IOERR_ROLLBACK_ATOMIC),
+      ADDINT(SQLITE_IOERR_COMMIT_ATOMIC),
+      ADDINT(SQLITE_IOERR_BEGIN_ATOMIC),
+      ADDINT(SQLITE_READONLY_CANTINIT),
+      ADDINT(SQLITE_ERROR_RETRY),
+      ADDINT(SQLITE_ERROR_MISSING_COLLSEQ),
+      ADDINT(SQLITE_READONLY_DIRECTORY),
+      ADDINT(SQLITE_LOCKED_VTAB),
+      ADDINT(SQLITE_CORRUPT_SEQUENCE),
       END,
 
       /* error codes */
@@ -1584,6 +1634,8 @@ modules etc. For example::
       ADDINT(SQLITE_CONFIG_PCACHE_HDRSZ),
       ADDINT(SQLITE_CONFIG_PMASZ),
       ADDINT(SQLITE_CONFIG_STMTJRNL_SPILL),
+      ADDINT(SQLITE_CONFIG_SMALL_MALLOC),
+      ADDINT(SQLITE_CONFIG_SORTERREF_SIZE),
       END,
 
       DICT("mapping_db_config"),
@@ -1592,6 +1644,15 @@ modules etc. For example::
       ADDINT(SQLITE_DBCONFIG_ENABLE_TRIGGER),
       ADDINT(SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER),
       ADDINT(SQLITE_DBCONFIG_ENABLE_LOAD_EXTENSION),
+      ADDINT(SQLITE_DBCONFIG_MAINDBNAME),
+      ADDINT(SQLITE_DBCONFIG_NO_CKPT_ON_CLOSE),
+      ADDINT(SQLITE_DBCONFIG_ENABLE_QPSG),
+      ADDINT(SQLITE_DBCONFIG_TRIGGER_EQP),
+#ifdef SQLITE_DBCONFIG_MAX
+      /* hopefully this constant will be removed */
+      ADDINT(SQLITE_DBCONFIG_MAX),
+#endif
+      ADDINT(SQLITE_DBCONFIG_RESET_DATABASE),
       END,
 
       DICT("mapping_status"),
@@ -1620,6 +1681,8 @@ modules etc. For example::
       ADDINT(SQLITE_DBSTATUS_CACHE_MISS),
       ADDINT(SQLITE_DBSTATUS_CACHE_WRITE),
       ADDINT(SQLITE_DBSTATUS_DEFERRED_FKS),
+      ADDINT(SQLITE_DBSTATUS_CACHE_USED_SHARED),
+      ADDINT(SQLITE_DBSTATUS_CACHE_SPILL),
       END,
 
       DICT("mapping_locking_level"),
@@ -1651,6 +1714,7 @@ modules etc. For example::
       ADDINT(SQLITE_IOCAP_UNDELETABLE_WHEN_OPEN),
       ADDINT(SQLITE_IOCAP_POWERSAFE_OVERWRITE),
       ADDINT(SQLITE_IOCAP_IMMUTABLE),
+      ADDINT(SQLITE_IOCAP_BATCH_ATOMIC),
       END,
 
       DICT("mapping_sync"),
@@ -1694,6 +1758,12 @@ modules etc. For example::
       ADDINT(SQLITE_FCNTL_ZIPVFS),
       ADDINT(SQLITE_FCNTL_JOURNAL_POINTER),
       ADDINT(SQLITE_FCNTL_VFS_POINTER),
+      ADDINT(SQLITE_FCNTL_WIN32_GET_HANDLE),
+      ADDINT(SQLITE_FCNTL_PDB),
+      ADDINT(SQLITE_FCNTL_COMMIT_ATOMIC_WRITE),
+      ADDINT(SQLITE_FCNTL_ROLLBACK_ATOMIC_WRITE),
+      ADDINT(SQLITE_FCNTL_BEGIN_ATOMIC_WRITE),
+      ADDINT(SQLITE_FCNTL_LOCK_TIMEOUT),
       END,
 
       DICT("mapping_conflict_resolution_modes"),
@@ -1765,6 +1835,7 @@ modules etc. For example::
  add_shell(m);
 
  PyModule_AddObject(m, "compile_options", get_compile_options());
+ PyModule_AddObject(m, "keywords", get_keywords());
 
  if(!PyErr_Occurred())
       {
