@@ -5,26 +5,26 @@
 */
 
 
-/** 
+/**
 .. _blobio:
 
 Blob Input/Output
 *****************
 
 A `blob <http://en.wikipedia.org/wiki/Binary_large_object>`_ is a
-SQLite `datatype <http://www.sqlite.org/datatype3.html>`_ representing
+SQLite `datatype <https://sqlite.org/datatype3.html>`_ representing
 a sequence of bytes.  It can be zero or more bytes in size.
 
 SQLite blobs have an absolute maximum size of 2GB and a `default
-maximum size <http://www.sqlite.org/c3ref/c_limit_attached.html>`_ of
+maximum size <https://sqlite.org/c3ref/c_limit_attached.html>`_ of
 1GB.
 
 An alternate approach to using blobs is to store the data in files and
 store the filename in the database.  Doing so loses the `ACID
-<http://www.sqlite.org/transactional.html>`_ properties of SQLite.
+<https://sqlite.org/transactional.html>`_ properties of SQLite.
 
 */
-   
+
 
 /* ZEROBLOB CODE */
 
@@ -32,7 +32,7 @@ store the filename in the database.  Doing so loses the `ACID
 
   If you want to insert a blob into a row, you previously needed to
   supply the entire blob in one go.  To read just one byte also
-  required retrieving the blob in its entireity. For example to insert
+  required retrieving the blob in its entirety. For example to insert
   a 100MB file you would have done::
 
      largedata=open("largefile", "rb").read()
@@ -77,7 +77,7 @@ ZeroBlobBind_init(ZeroBlobBind *self, PyObject *args, PyObject *kwargs)
       PyErr_Format(PyExc_TypeError, "Zeroblob constructor does not take keyword arguments");
       return -1;
     }
-  
+
   if(!PyArg_ParseTuple(args, "i", &n))
     return -1;
 
@@ -92,7 +92,7 @@ ZeroBlobBind_init(ZeroBlobBind *self, PyObject *args, PyObject *kwargs)
 }
 
 /** .. method:: length() -> int
- 
+
   Size of zero blob in bytes.
 */
 static PyObject *
@@ -112,7 +112,7 @@ static PyTypeObject ZeroBlobBindType = {
     "apsw.zeroblob",           /*tp_name*/
     sizeof(ZeroBlobBind),      /*tp_basicsize*/
     0,                         /*tp_itemsize*/
-    0,                         /*tp_dealloc*/ 
+    0,                         /*tp_dealloc*/
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
@@ -180,14 +180,14 @@ static PyTypeObject APSWBlobType;
   This object is created by :meth:`Connection.blobopen` and provides
   access to a blob in the database.  It behaves like a Python file.
   At the C level it wraps a `sqlite3_blob
-  <http://sqlite.org/c3ref/blob.html>`_.
+  <https://sqlite.org/c3ref/blob.html>`_.
 
   .. note::
 
     You cannot change the size of a blob using this object. You should
     create it with the correct size in advance either by using
     :class:`zeroblob` or the `zeroblob()
-    <http://www.sqlite.org/lang_corefunc.html>`_ function.
+    <https://sqlite.org/lang_corefunc.html>`_ function.
 
   See the :ref:`example <example-blobio>`.
 */
@@ -210,7 +210,7 @@ APSWBlob_close_internal(APSWBlob *self, int force)
   PyObject *err_type, *err_value, *err_traceback;
 
   if(force==2)
-    PyErr_Fetch(&err_type, &err_value, &err_traceback);    
+    PyErr_Fetch(&err_type, &err_value, &err_traceback);
 
   /* note that sqlite3_blob_close always works even if an error is
      returned - see sqlite ticket #2815 */
@@ -231,8 +231,8 @@ APSWBlob_close_internal(APSWBlob *self, int force)
               break;
             case 2:
               SET_EXC(res, self->connection->db);
-              apsw_write_unraiseable(NULL);            
-            }         
+              apsw_write_unraiseable(NULL);
+            }
         }
       self->pBlob=0;
     }
@@ -242,7 +242,7 @@ APSWBlob_close_internal(APSWBlob *self, int force)
      we'd still be in list */
   if(self->connection)
     Connection_remove_dependent(self->connection, (PyObject*)self);
-  
+
   Py_CLEAR(self->connection);
 
   if(force==2)
@@ -270,7 +270,7 @@ APSWBlob_dealloc(APSWBlob *self)
 
 /** .. method:: length() -> int
 
-  Returns the size of the blob in bytes. 
+  Returns the size of the blob in bytes.
 
   -* sqlite3_blob_bytes
 */
@@ -305,7 +305,7 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
 
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
-   
+
   /* The python file read routine treats negative numbers as read till
      end of file, which I think is rather silly.  (Try reading -3
      bytes from /dev/zero on a 64 bit machine with lots of swap to see
@@ -334,6 +334,8 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
 
   thebuffer= PyBytes_AS_STRING(buffy);
   PYSQLITE_BLOB_CALL(res=sqlite3_blob_read(self->pBlob, thebuffer, length, self->curoffset));
+  if(PyErr_Occurred())
+    return NULL;
 
   if(res!=SQLITE_OK)
     {
@@ -430,6 +432,8 @@ APSWBlob_readinto(APSWBlob *self, PyObject *args)
     return PyErr_Format(PyExc_ValueError, "More data requested than blob length");
 
   PYSQLITE_BLOB_CALL(res=sqlite3_blob_read(self->pBlob, (char*)buffer+offset, lengthwanted, self->curoffset));
+  if(PyErr_Occurred())
+    return NULL;
 
   if(res!=SQLITE_OK)
     {
@@ -446,10 +450,10 @@ APSWBlob_readinto(APSWBlob *self, PyObject *args)
   Changes current position to *offset* biased by *whence*.
 
   :param offset: New position to seek to.  Can be positive or negative number.
-  :param whence: Use 0 if *offset* is relative to the begining of the blob,
+  :param whence: Use 0 if *offset* is relative to the beginning of the blob,
                  1 if *offset* is relative to the current position,
                  and 2 if *offset* is relative to the end of the blob.
-  :raises ValueError: If the resulting offset is before the begining (less than zero) or beyond the end of the blob.
+  :raises ValueError: If the resulting offset is before the beginning (less than zero) or beyond the end of the blob.
 */
 
 static PyObject *
@@ -458,15 +462,15 @@ APSWBlob_seek(APSWBlob *self, PyObject *args)
   int offset, whence=0;
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
-  
+
   if(!PyArg_ParseTuple(args, "i|i:seek(offset,whence=0)", &offset, &whence))
     return NULL;
-  
+
   switch(whence)
     {
     default:
       return PyErr_Format(PyExc_ValueError, "whence parameter should be 0, 1 or 2");
-    case 0: /* relative to begining of file */
+    case 0: /* relative to beginning of file */
       if(offset<0 || offset>sqlite3_blob_bytes(self->pBlob))
         goto out_of_range;
       self->curoffset=offset;
@@ -544,6 +548,8 @@ APSWBlob_write(APSWBlob *self, PyObject *obj)
     return PyErr_Format(PyExc_ValueError, "Data would go beyond end of blob");
 
   PYSQLITE_BLOB_CALL(res=sqlite3_blob_write(self->pBlob, buffer, size, self->curoffset));
+  if(PyErr_Occurred())
+    return NULL;
 
   if(res!=SQLITE_OK)
     {
@@ -559,7 +565,7 @@ APSWBlob_write(APSWBlob *self, PyObject *obj)
 /** .. method:: close([force=False])
 
   Closes the blob.  Note that even if an error occurs the blob is
-  still closed (see :cvstrac:`2815`).
+  still closed.
 
   .. note::
 
@@ -597,7 +603,7 @@ APSWBlob_close(APSWBlob *self, PyObject *args)
   Py_RETURN_NONE;
 }
 
-/** .. method:: __enter__() -> context 
+/** .. method:: __enter__() -> context
 
   You can use a blob as a `context manager
   <http://docs.python.org/reference/datamodel.html#with-statement-context-managers>`_
@@ -644,7 +650,7 @@ APSWBlob_exit(APSWBlob *self, APSW_ARGUNUSED PyObject *args)
   Py_RETURN_FALSE;
 }
 
-/** .. method:: reopen(rowid) 
+/** .. method:: reopen(rowid)
 
   Change this blob object to point to a different row.  It can be
   faster than closing an existing blob an opening a new one.
@@ -674,11 +680,14 @@ APSWBlob_reopen(APSWBlob *self, PyObject *arg)
       }
     else
       return PyErr_Format(PyExc_TypeError, "blob reopen argument must be a number");
-  
+
   /* no matter what happens we always reset current offset */
   self->curoffset=0;
 
   PYSQLITE_BLOB_CALL(res=sqlite3_blob_reopen(self->pBlob, rowid));
+  if(PyErr_Occurred())
+    return NULL;
+
   if(res!=SQLITE_OK)
     {
       SET_EXC(res, self->connection->db);
@@ -716,7 +725,7 @@ static PyTypeObject APSWBlobType = {
     "apsw.blob",               /*tp_name*/
     sizeof(APSWBlob),          /*tp_basicsize*/
     0,                         /*tp_itemsize*/
-    (destructor)APSWBlob_dealloc, /*tp_dealloc*/ 
+    (destructor)APSWBlob_dealloc, /*tp_dealloc*/
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
@@ -760,4 +769,3 @@ static PyTypeObject APSWBlobType = {
     0                          /* tp_del */
     APSW_PYTYPE_VERSION
 };
-
